from termcolor import colored
from ..utils.constants import LOGConstants
import colorama


class ConsoleLogger(object):
    def __init__(self):
        """Initialization constructor WidgetLogger"""
        colorama.init()
        pass

    def _get_color_for_label(self, label):
        """Get color from given label

        Args:
          label: 

        Returns:

        """
        log_attributes = [a for a in dir(LOGConstants) if not a.startswith("__")]
        attribute_values = [[LOGConstants.__dict__[key]] for key in LOGConstants.__dict__.keys() if
                            key in log_attributes]
        color = [i[0][1] for i in attribute_values if i[0][0] == label][0]
        return color

    def emit(self, label, record):
        """emit/print the message

        Args:
          label: 
          record: 

        Returns:

        """
        color = self._get_color_for_label(label)
        print(colored(label, color) + "\t" + record)



import os

from ..utils.constants import LOGConstants, LoggerSCRIPT, IsommeConstants, UnitsConstants, TransducerConstants


class ISOMMEConverter:
    """ """
    def __init__(self):
        self._unit_list = []
        self._transducer_type_list = []
        self._channel_values = []
        self._channel_codes = []
        self._channel_names = []
        self._year_of_test = ""
        self._name_of_test_object = ""
        self._test_nr = ""
        self._model_info = ""
        self._file_directory = ""
        self._file_name = ""
        self._time_of_first_sample = ""

    def write_ISOMME(self, path_to_dir=None, data=None, dynasaur_definitions=None, logger=None, test=False):
        """

        Args:
          path_to_dir:  (Default value = None)
          data:  (Default value = None)
          dynasaur_definitions:  (Default value = None)
          logger:  (Default value = None)
          test:  (Default value = False)

        Returns:

        """
        self._file_directory = path_to_dir
        if isinstance(self._file_directory, str) and self._file_directory == 'exit':
            return -1
        if not os.path.isdir(self._file_directory):
            logger.emit(LOGConstants.ERROR[0], "File directory doesn't exist.")

        check_path = [True, True, True, True]

        if not test:
            while any(check_path):
                if check_path[0]:
                    self._model_info = input('\033[36m' + LOGConstants.INPUT[0]
                                             + '\033[0m' + '\t' "Please enter if your model is AM50 or 6yo: ")
                    if self._model_info == IsommeConstants.MODEL_AM50 or self._model_info == IsommeConstants.MODEL_6YO:
                        check_path[0] = False
                    else:
                        logger.emit(LOGConstants.ERROR[0], "Model doesn't exist.")
                if check_path[1]:
                    self._year_of_test = input('\033[36m' + LOGConstants.INPUT[
                        0] + '\033[0m' + '\t' "Please enter the year of the test (eg: 2019): ")
                    if len(self._year_of_test) == 4:
                        check_path[1] = False
                    else:
                        logger.emit(LOGConstants.ERROR[0], "Date has the wrong format.")
                if check_path[2]:
                    self._name_of_test_object = input('\033[36m' + LOGConstants.INPUT[
                        0] + '\033[0m' + '\t' "Please enter the name of the test object (e.g Pedestrian Model): ")
                    if self._name_of_test_object != "":
                        check_path[2] = False
                    else:
                        logger.emit(LOGConstants.ERROR[0], "No name entered.")
                if check_path[3]:
                    add_info = input('\033[36m' + LOGConstants.INPUT[
                        0] + '\033[0m' + '\t' "Please enter the model information (e.g FCR_30kph): ")
                    if add_info != "":
                        self._test_nr = add_info + "_" + self._model_info
                        check_path[3] = False
                    else:
                        logger.emit(LOGConstants.ERROR[0], "No test number entered.")
            else:
                self._model_info = IsommeConstants.MODEL_AM50
                self._year_of_test = 2020
                self._name_of_test_object = "Model"
                self._test_nr = "info" + "_" + self._model_info

        identifier_list = [tuple(x.split(":")) if x is not None else (None, None, None) for x in
                           dynasaur_definitions.get_info()]
        identifier_list.insert(0, (IsommeConstants.TIME, None, None))

        check_append = True
        for part_of in data.keys():
            for diagram_name in data[part_of].keys():
                if check_append:
                    self._channel_values.append(data[part_of][diagram_name]["Y"])
                    check_append = False
                self._channel_values.append(data[part_of][diagram_name]["X"])

        self._channel_codes = self.get_channel_codes(identifier_list)
        self._channel_names = self.get_channel_names(identifier_list)

        self._unit_list = self.get_unit_type_list(identifier_list)

        self._transducer_type_list = self.get_transducer_type(self._unit_list)

        self._time_of_first_sample = data[list(data.keys())[0]][list(data[list(data.keys())[0]].keys())[0]]["Y"]
        channel_dict_list = self.get_channel_information_dict()

        nr_of_channels = len(data[list(data.keys())[0]][list(data[list(data.keys())[0]].keys())[0]][
                                 "X"])  # len([channel for channel in channel_values if channel != None])

        self.create_output_directory(self._file_directory)
        channel_summary_dict = self.get_channel_summary_dict(nr_of_channels)
        test_nr_mme_dict = self.get_test_nr_mme_dict()

        logger.emit(LOGConstants.SCRIPT[0],
                    LoggerSCRIPT.print_statements[3] + os.path.join(self._file_directory, self._test_nr, "Channel"))
        self.print_channels(channel_dict_list)

        self.print_channel_summary(channel_summary_dict)

        logger.emit(LOGConstants.SCRIPT[0],
                    LoggerSCRIPT.print_statements[5] + os.path.join(self._file_directory, self._test_nr))
        self.print_mme(test_nr_mme_dict)

    def get_channel_information_dict(self):
        """ """
        channel_dict_list = []

        for i, channel_name in enumerate(self._channel_names):
            nr_of_samples = str(len(self._channel_values[i]))

            channel_dict = {"Test object number": 1, "Name of the channel": channel_name,
                            "Laboratory channel code": IsommeConstants.NO_VALUE,
                            "Customer channel code": IsommeConstants.NO_VALUE,
                            "Channel code": self._channel_codes[i], "Unit": self._unit_list[i],
                            "Reference system": IsommeConstants.NO_VALUE,
                            "Transducer type": self._transducer_type_list[i],
                            "Pre filter type": IsommeConstants.NO_VALUE, "Cut off frequency": IsommeConstants.NO_VALUE,
                            "Channel amplitude class": IsommeConstants.NO_VALUE,
                            "Sampling interval": IsommeConstants.NO_VALUE,
                            "Bit resolution": IsommeConstants.NO_VALUE,
                            "Time of first sample": self._time_of_first_sample,
                            "Number of samples": nr_of_samples}

            channel_dict_list.append(channel_dict)

        return channel_dict_list

    def get_channel_summary_dict(self, nr_of_channels):
        """

        Args:
          nr_of_channels: 

        Returns:

        """
        channel_summary_dict = {"Instrumentation standard": IsommeConstants.INSTUMENTATION_STANDARD,
                                "Number of channels": str(nr_of_channels)}
        for i, channel in enumerate(self._channel_names):
            channel_nr_string = str(i + 1)
            channel_summary_dict["Name of channel " + channel_nr_string.rjust(3, '0')] = self._channel_codes[
                                                                                             i] + " / " + channel

        return channel_summary_dict

    def get_test_nr_mme_dict(self):
        """ """
        mme_dict = {"Data format edition number": "1.6", "Customer name": "Euro NCAP",
                    "Customer test ref number": "1", "Title": "Euro NCAP " + self._year_of_test,
                    "Type of the test": "Pedestrian", "Subtype of the test": "Certification",
                    "Regulation": IsommeConstants.NO_VALUE,
                    "Name of test object 1": self._name_of_test_object,
                    "Class of test object 1": IsommeConstants.NO_VALUE,
                    "Ref. number of test object 1": IsommeConstants.NO_VALUE, "Size": self._model_info}
        return mme_dict

    def get_channel_names(self, identifier_list):
        """

        Args:
          identifier_list: 

        Returns:

        """
        channel_names = []
        for identifier in identifier_list:
            channel_name = ""
            for j, item in enumerate(identifier):
                if j % 3 != 0:
                    if item is not None and item != "":
                        channel_name += item + "_"
            channel_name = channel_name.strip("_")
            channel_names.append(channel_name)
        return channel_names

    def get_unit_type_list(self, identifier_list):
        """

        Args:
          identifier_list: 

        Returns:

        """
        unit_type_list = []
        for identifier in identifier_list:
            if identifier[0] == IsommeConstants.TIME:
                unit_type_list.append(UnitsConstants.MILLISECOND)
            elif identifier[0] == IsommeConstants.CONTACT_FORCE:
                unit_type_list.append(UnitsConstants.KNEWTON)
            elif identifier[0] == IsommeConstants.TRAJECTORY:
                unit_type_list.append(UnitsConstants.MILLIMETER)
            elif identifier[0] == IsommeConstants.HEAD_COG:
                unit_type_list.append("g")
            elif identifier[0] == IsommeConstants.HEAD_COG and identifier[1] == "velocity":
                unit_type_list.append(UnitsConstants.VELOCITY)
            elif identifier[0] == IsommeConstants.ENERGY:
                unit_type_list.append(UnitsConstants.JOULE)
            elif identifier[0] == IsommeConstants.ADDED_MASS:
                unit_type_list.append(UnitsConstants.KILOGRAM)
            elif identifier[0] == IsommeConstants.TIMESTEP:
                unit_type_list.append(UnitsConstants.MILLISECOND)
            elif identifier[0] == IsommeConstants.GV:
                unit_type_list.append(UnitsConstants.MILLIMETER)
            elif identifier[0] == IsommeConstants.ENERGY_HBM:
                unit_type_list.append(UnitsConstants.JOULE)
            else:
                unit_type_list.append(IsommeConstants.NO_VALUE)
        return unit_type_list

    def get_transducer_type(self, unit_list):
        """

        Args:
          unit_list: 

        Returns:

        """
        transducer_type_list = []
        for unit in unit_list:
            if unit == UnitsConstants.MILLIMETER:
                transducer_type_list.append(TransducerConstants.COORDINATE)
            elif unit == UnitsConstants.KNEWTON:
                transducer_type_list.append(TransducerConstants.FORCE)
            elif unit == UnitsConstants.MILLISECOND:
                transducer_type_list.append(TransducerConstants.TIME)
            elif unit == "g":
                transducer_type_list.append(TransducerConstants.ACCELERATION)
            elif unit == UnitsConstants.VELOCITY:
                transducer_type_list.append(TransducerConstants.VELOCITY)
            elif unit == UnitsConstants.JOULE:
                transducer_type_list.append(TransducerConstants.ENERGY)
            elif unit == UnitsConstants.KILOGRAM:
                transducer_type_list.append(TransducerConstants.MASS)
            elif unit == "%":
                transducer_type_list.append(TransducerConstants.MASS)
            else:
                transducer_type_list.append(IsommeConstants.NO_VALUE)
        return transducer_type_list

    def get_channel_codes(self, header_info):
        """

        Args:
          header_info: 

        Returns:

        """
        channel_code = []
        if self._model_info == IsommeConstants.MODEL_6YO:
            for i, info in enumerate(header_info):
                ###############TIME##################
                if info[0] == IsommeConstants.TIME:
                    channel_code.append("??TIRS??????TI?0")
                ###############COORDINATES/TRAJECTORIES##################
                elif info[1] == IsommeConstants.HC and info[2] == IsommeConstants.X_COORDINATE:
                    channel_code.append("??HEADHC00V6COX0")
                elif info[1] == IsommeConstants.HC and info[2] == IsommeConstants.Z_COORDINATE:
                    channel_code.append("??HEADHC00V6COZ0")
                elif info[1] == IsommeConstants.C7 and info[2] == IsommeConstants.X_COORDINATE:
                    channel_code.append("??CESP0700V6COX0")
                elif info[1] == IsommeConstants.C7 and info[2] == IsommeConstants.Z_COORDINATE:
                    channel_code.append("??CESP0700V6COZ0")
                elif info[1] == IsommeConstants.T12 and info[2] == IsommeConstants.X_COORDINATE:
                    channel_code.append("??THSP1200V6COX0")
                elif info[1] == IsommeConstants.T12 and info[2] == IsommeConstants.Z_COORDINATE:
                    channel_code.append("??THSP1200V6COZ0")
                elif info[1] == IsommeConstants.AC and info[2] == IsommeConstants.X_COORDINATE:
                    channel_code.append("??ACTBAC00V6COX0")
                elif info[1] == IsommeConstants.AC and info[2] == IsommeConstants.Z_COORDINATE:
                    channel_code.append("??ACTBAC00V6COZ0")
                elif info[1] == IsommeConstants.T8 and info[2] == IsommeConstants.X_COORDINATE:
                    channel_code.append("??THSP0800V6COX0")
                elif info[1] == IsommeConstants.T8 and info[2] == IsommeConstants.Z_COORDINATE:
                    channel_code.append("??THSP0800V6COZ0")
                elif info[1] == IsommeConstants.FER and info[2] == IsommeConstants.X_COORDINATE:
                    channel_code.append("??FEMRFR00V6COX0")
                elif info[1] == IsommeConstants.FER and info[2] == IsommeConstants.Z_COORDINATE:
                    channel_code.append("??FEMRFR00V6COZ0")
                elif info[1] == IsommeConstants.MR and info[2] == IsommeConstants.X_COORDINATE:
                    channel_code.append("??ANKLMR00V6COX0")
                elif info[1] == IsommeConstants.MR and info[2] == IsommeConstants.Z_COORDINATE:
                    channel_code.append("??ANKLMR00V6COZ0")
                elif info[1] == IsommeConstants.FEL and info[2] == IsommeConstants.X_COORDINATE:
                    channel_code.append("??FEMRFL00V6COX0")
                elif info[1] == IsommeConstants.FEL and info[2] == IsommeConstants.Z_COORDINATE:
                    channel_code.append("??FEMRFL00V6COZ0")
                elif info[1] == IsommeConstants.ML and info[2] == IsommeConstants.X_COORDINATE:
                    channel_code.append("??ANKLML00V6COX0")
                elif info[1] == IsommeConstants.ML and info[2] == IsommeConstants.Z_COORDINATE:
                    channel_code.append("??ANKLML00V6COZ0")
                elif info[1] == IsommeConstants.FEL and info[2] == IsommeConstants.X_COORDINATE:
                    channel_code.append("??FEMRFEL00V6COX0")
                elif info[1] == IsommeConstants.FEL and info[2] == IsommeConstants.Z_COORDINATE:
                    channel_code.append("??FEMRFEL00V6COZ0")
                ###############RESULTANTS##################
                elif info[1] == IsommeConstants.PEDESTRIAN_GV and info[2] == IsommeConstants.RESULTANT_CF:
                    channel_code.append("10VEHCSU00V6FOR0")
                elif info[1] == IsommeConstants.HEAD_GV and info[2] == IsommeConstants.RESULTANT_CF:
                    channel_code.append("10HEAD0700V6FOR0")
                elif info[1] == IsommeConstants.ARM_GV and info[2] == IsommeConstants.RESULTANT_CF:
                    channel_code.append("10VEHC4500V6FOR0")
                elif info[1] == IsommeConstants.RIGHT_LEG_BUMPER and info[2] == IsommeConstants.RESULTANT_CF:
                    channel_code.append("10BUMP0200V6FOR0")
                elif info[1] == IsommeConstants.TORSO_BUMPER and info[2] == IsommeConstants.RESULTANT_CF:
                    channel_code.append("10BUMP0700V6FOR0")
                elif info[1] == IsommeConstants.PEDESTRIAN_BUMPER and info[2] == IsommeConstants.RESULTANT_CF:
                    channel_code.append("10BUMPSU00V6FOR0")
                elif info[1] == IsommeConstants.PEDESTRIAN_BONNET and info[2] == IsommeConstants.RESULTANT_CF:
                    channel_code.append("10HOODSU00V6FOR0")
                ###############Energies##################
                elif info[1] == "" and info[2] == IsommeConstants.TOTAL_HOURGLASS_ENERGY:
                    channel_code.append("00EHOUSU00V6EN00")
                elif info[1] == "" and info[2] == IsommeConstants.TOTAL_INTERNAL_ENERGY:
                    channel_code.append("00EINTSU00V6EN00")
                elif info[1] == "" and info[2] == IsommeConstants.TOTAL_ENERGY:
                    channel_code.append("00ETOTSU00V6EN00")
                elif info[1] == "" and info[2] == IsommeConstants.CONTACT_ENERGY:
                    channel_code.append("00ESLISU00V6EN00")
                ###############HEAD COG##################
                elif info[1] == "" and info[2] == IsommeConstants.Z_ACCELERATION:
                    channel_code.append("??HEADHC00V6ACZ0")
                elif info[1] == "" and info[2] == IsommeConstants.RESULTANT_ACCELERATION:
                    channel_code.append("??HEADHC00V6ACR0")
                elif info[1] == "" and info[2] == IsommeConstants.RESULTANT_VELOCITY:
                    channel_code.append("??HEADHC00V6VER0")
                ###############MASS##################
                elif info[1] == IsommeConstants.ADDED_MASS_WS and info[2] == IsommeConstants.WHOLE_SETUP:
                    channel_code.append("00MINCSU00V6MA00")
                elif info[1] == IsommeConstants.REL_ADDED_MASS_WS and info[2] == IsommeConstants.WHOLE_SETUP:
                    channel_code.append("00MINCSU00V6SE00")
                ###############TIMESTEP##################
                elif info[0] == IsommeConstants.TIMESTEP and info[1] == "" and info[2] == "":
                    channel_code.append("00DTIMSU00V6TI00")
                ###############GV##################
                elif info[0] == IsommeConstants.GV and info[2] == IsommeConstants.X_COORDINATE:
                    channel_code.append("10VEHCCG00V6CO00")
                ###############TOTAL HOURGLASS ENERGY HBM##################
                elif info[1] == IsommeConstants.TOTAL_HOURGLASS_ENERGY_HBM and info[2] == "":
                    channel_code.append("??EHOUSU00V6EN00")
                ###############ADDED MASS HBM##################
                elif info[1] == IsommeConstants.ADDED_MASS_WS and info[2] == IsommeConstants.HBM:
                    channel_code.append("??MINCSU00V6MA00")
                ###############HIT##################
                elif info[1] == IsommeConstants.HIT or info[2] == IsommeConstants.HIT or info[0] == IsommeConstants.HIT:
                    channel_code.append("??HEAD??DAV6TI?0")
                ###############WAD##################
                elif info[1] == IsommeConstants.WAD or info[2] == IsommeConstants.WAD or info[0] == IsommeConstants.WAD:
                    channel_code.append("??HEAD??DCV6TI?0")
                else:
                    channel_code.append(IsommeConstants.NO_VALUE)
            return channel_code
        elif self._model_info == IsommeConstants.MODEL_AM50:
            for i, info in enumerate(header_info):
                ###############TIME##################
                if info[0] == IsommeConstants.TIME:
                    channel_code.append("??TIRS??????TI?0")
                ###############COORDINATES/TRAJECTORIES##################
                elif info[1] == IsommeConstants.HC and info[2] == IsommeConstants.X_COORDINATE:
                    channel_code.append("??HEADHC00VHCOX0")
                elif info[1] == IsommeConstants.HC and info[2] == IsommeConstants.Z_COORDINATE:
                    channel_code.append("??HEADHC00VHCOZ0")
                elif info[1] == IsommeConstants.C7 and info[2] == IsommeConstants.X_COORDINATE:
                    channel_code.append("??CESP0700VHCOX0")
                elif info[1] == IsommeConstants.C7 and info[2] == IsommeConstants.Z_COORDINATE:
                    channel_code.append("??CESP0700VHCOZ0")
                elif info[1] == IsommeConstants.T12 and info[2] == IsommeConstants.X_COORDINATE:
                    channel_code.append("??THSP1200VHCOX0")
                elif info[1] == IsommeConstants.T12 and info[2] == IsommeConstants.Z_COORDINATE:
                    channel_code.append("??THSP1200VHCOZ0")
                elif info[1] == IsommeConstants.AC and info[2] == IsommeConstants.X_COORDINATE:
                    channel_code.append("??ACTBAC00VHCOX0")
                elif info[1] == IsommeConstants.AC and info[2] == IsommeConstants.Z_COORDINATE:
                    channel_code.append("??ACTBAC00VHCOZ0")
                elif info[1] == IsommeConstants.T8 and info[2] == IsommeConstants.X_COORDINATE:
                    channel_code.append("??THSP0800VHCOX0")
                elif info[1] == IsommeConstants.T8 and info[2] == IsommeConstants.Z_COORDINATE:
                    channel_code.append("??THSP0800VHCOZ0")
                elif info[1] == IsommeConstants.FER and info[2] == IsommeConstants.X_COORDINATE:
                    channel_code.append("??FEMRFR00VHCOX0")
                elif info[1] == IsommeConstants.FER and info[2] == IsommeConstants.Z_COORDINATE:
                    channel_code.append("??FEMRFR00VHCOZ0")
                elif info[1] == IsommeConstants.MR and info[2] == IsommeConstants.X_COORDINATE:
                    channel_code.append("??ANKLMR00VHCOX0")
                elif info[1] == IsommeConstants.MR and info[2] == IsommeConstants.Z_COORDINATE:
                    channel_code.append("??ANKLMR00VHCOZ0")
                elif info[1] == IsommeConstants.FEL and info[2] == IsommeConstants.X_COORDINATE:
                    channel_code.append("??FEMRFL00VHCOX0")
                elif info[1] == IsommeConstants.FEL and info[2] == IsommeConstants.Z_COORDINATE:
                    channel_code.append("??FEMRFL00VHCOZ0")
                elif info[1] == IsommeConstants.ML and info[2] == IsommeConstants.X_COORDINATE:
                    channel_code.append("??ANKLML00VHCOX0")
                elif info[1] == IsommeConstants.ML and info[2] == IsommeConstants.Z_COORDINATE:
                    channel_code.append("??ANKLML00VHCOZ0")
                elif info[1] == IsommeConstants.FEL and info[2] == IsommeConstants.X_COORDINATE:
                    channel_code.append("??FEMRFEL00VHCOX0")
                elif info[1] == IsommeConstants.FEL and info[2] == IsommeConstants.Z_COORDINATE:
                    channel_code.append("??FEMRFEL00VHCOZ0")
                ###############RESULTANTS##################
                elif info[1] == IsommeConstants.PEDESTRIAN_GV and info[2] == IsommeConstants.RESULTANT_CF:
                    channel_code.append("10VEHCSU00VHFOR0")
                elif info[1] == IsommeConstants.HEAD_GV and info[2] == IsommeConstants.RESULTANT_CF:
                    channel_code.append("10HEAD0700VHFOR0")
                elif info[1] == IsommeConstants.ARM_GV and info[2] == IsommeConstants.RESULTANT_CF:
                    channel_code.append("10VEHC4500VHFOR0")
                elif info[1] == IsommeConstants.RIGHT_LEG_BUMPER and info[2] == IsommeConstants.RESULTANT_CF:
                    channel_code.append("10BUMP0200VHFOR0")
                elif info[1] == IsommeConstants.TORSO_BUMPER and info[2] == IsommeConstants.RESULTANT_CF:
                    channel_code.append("10BUMP0700VHFOR0")
                elif info[1] == IsommeConstants.PEDESTRIAN_BUMPER and info[2] == IsommeConstants.RESULTANT_CF:
                    channel_code.append("10BUMPSU00VHFOR0")
                elif info[1] == IsommeConstants.PEDESTRIAN_BONNET and info[2] == IsommeConstants.RESULTANT_CF:
                    channel_code.append("10HOODSU00VHFOR0")
                ###############Energies##################
                elif info[1] == "" and info[2] == IsommeConstants.TOTAL_HOURGLASS_ENERGY:
                    channel_code.append("00EHOUSU00VHEN00")
                elif info[1] == "" and info[2] == IsommeConstants.TOTAL_INTERNAL_ENERGY:
                    channel_code.append("00EINTSU00VHEN00")
                elif info[1] == "" and info[2] == IsommeConstants.TOTAL_ENERGY:
                    channel_code.append("00ETOTSU00VHEN00")
                elif info[1] == "" and info[2] == IsommeConstants.CONTACT_ENERGY:
                    channel_code.append("00ESLISU00VHEN00")
                ###############HEAD COG##################
                elif info[1] == "" and info[2] == IsommeConstants.Z_ACCELERATION:
                    channel_code.append("??HEADHC00VHACZ0")
                elif info[1] == "" and info[2] == IsommeConstants.RESULTANT_ACCELERATION:
                    channel_code.append("??HEADHC00VHACR0")
                elif info[1] == "" and info[2] == IsommeConstants.RESULTANT_VELOCITY:
                    channel_code.append("??HEADHC00VHVER0")
                ###############MASS##################
                elif info[1] == IsommeConstants.ADDED_MASS_WS and info[2] == IsommeConstants.WHOLE_SETUP:
                    channel_code.append("00MINCSU00VHMA00")
                elif info[1] == IsommeConstants.REL_ADDED_MASS_WS and info[2] == IsommeConstants.WHOLE_SETUP:
                    channel_code.append("00MINCSU00VHSE00")
                ###############TIMESTEP##################
                elif info[0] == IsommeConstants.TIMESTEP and info[1] == "" and info[2] == "":
                    channel_code.append("00DTIMSU00VHTI00")
                ###############GV##################
                elif info[0] == IsommeConstants.GV and info[2] == IsommeConstants.X_COORDINATE:
                    channel_code.append("10VEHCCG00VHCO00")
                ###############TOTAL HOURGLASS ENERGY HBM##################
                elif info[1] == IsommeConstants.TOTAL_HOURGLASS_ENERGY_HBM and info[2] == "":
                    channel_code.append("??EHOUSU00VHEN00")
                ###############ADDED MASS HBM##################
                elif info[1] == IsommeConstants.ADDED_MASS_WS and info[2] == IsommeConstants.HBM:
                    channel_code.append("??MINCSU00VHMA00")
                ###############HIT##################
                elif info[1] == IsommeConstants.HIT or info[2] == IsommeConstants.HIT or info[0] == IsommeConstants.HIT:
                    channel_code.append("??HEAD??DAVHTI?0")
                ###############WAD##################
                elif info[1] == IsommeConstants.WAD or info[2] == IsommeConstants.WAD or info[0] == IsommeConstants.WAD:
                    channel_code.append("??HEAD??DCVHTI?0")
                else:
                    channel_code.append(IsommeConstants.NO_VALUE)
            return channel_code
        else:
            return [IsommeConstants.NO_VALUE] * len(header_info)

    def print_channels(self, channel_dict_list):
        """

        Args:
          channel_dict_list: 

        Returns:

        """
        for i, channel_dict in enumerate(channel_dict_list):
            channel_nr_string = str(i + 1)
            path = os.path.join(self._file_directory, self._test_nr, "Channel",
                                self._test_nr + "." + channel_nr_string.rjust(3, '0'))
            with open(path, 'w') as channel_file:
                for key, value in channel_dict.items():
                    nr_whitespaces = 28 - len(key)
                    whitespaces = " " * nr_whitespaces
                    row = key + whitespaces + ":" + str(value)
                    channel_file.write('{}\n'.format(row))
                for value in self._channel_values[i]:
                    channel_file.write('{}\n'.format(value))

    def print_channel_summary(self, channel_summary_dict):
        """

        Args:
          channel_summary_dict: 

        Returns:

        """
        path = os.path.join(self._file_directory, self._test_nr, "Channel", self._test_nr + ".chn")
        with open(path, 'w') as channel_summary_file:
            for key, value in channel_summary_dict.items():
                nr_whitespaces = 28 - len(key)
                whitespaces = " " * nr_whitespaces
                row = key + whitespaces + ":" + value
                channel_summary_file.write('{}\n'.format(row))

    def print_mme(self, mme_dict):
        """

        Args:
          mme_dict: 

        Returns:

        """
        path = os.path.join(self._file_directory, self._test_nr, self._test_nr + ".mme")
        with open(path, 'w') as mme_file:
            for key, value in mme_dict.items():
                nr_whitespaces = 28 - len(key)
                whitespaces = " " * nr_whitespaces
                row = key + whitespaces + ":" + value
                mme_file.write('{}\n'.format(row))

    def create_output_directory(self, output_directory):
        """

        Args:
          output_directory: 

        Returns:

        """
        if not os.path.exists(os.path.join(output_directory, self._test_nr)):
            os.mkdir(os.path.join(output_directory, self._test_nr))
        if not os.path.exists(os.path.join(output_directory, self._test_nr, "Channel")):
            os.mkdir(os.path.join(output_directory, self._test_nr, "Channel"))




from ..data.ls_dyna import DEForc, Elout, EloutObject, Glstat, Matsum, Nodout, RCForc, SBTout, SlipringOut, Secforc, Disbout, \
                           PartDisbout, Abstat, AbstatCPM, JointForc, Sleout, RigidBody, BoundaryCondition, JointType
from ..utils.constants import ObjectConstantsForData


class DataContainer:
    """class DataContainer
    store objects types

    Args:

    Returns:

    """
    _elout = None
    _elout_object = None
    _secforc = None
    _rcforc = None
    _deforc = None
    _nodout = None
    _joint_forc = None
    _joint_type = None
    _volume = None
    _sbtout = None
    _slipringout = None
    _matsum = None
    _glstat = None
    _disbout = None
    _part_disbout = None
    _abstat = None
    _abstat_cpm = None
    _sleout = None
    _rigid_body = None
    _boundary_condition = None

    @staticmethod
    def get_data(name):
        """

        Args:
          name: 

        Returns:
          Object on the given name

        """
        if name == ObjectConstantsForData.ELEMENT:
            return DataContainer._elout
        elif name == ObjectConstantsForData.ELEMENTOBJECT:
            return DataContainer._elout_object
        elif name == ObjectConstantsForData.CROSS_SECTION:
            return DataContainer._secforc
        elif name == ObjectConstantsForData.DISCRETE:
            return DataContainer._deforc
        elif name == ObjectConstantsForData.NODE:
            return DataContainer._nodout
        elif name == ObjectConstantsForData.JOINT:
            return DataContainer._joint_forc
        elif name == ObjectConstantsForData.JOINT_TYPE:
            return DataContainer._joint_type
        elif name == ObjectConstantsForData.SEAT_BELT:
            return DataContainer._sbtout
        elif name == ObjectConstantsForData.SLIP_RING:
            return DataContainer._slipringout
        elif name == ObjectConstantsForData.CONTACT:
            return DataContainer._rcforc
        elif name == ObjectConstantsForData.ENERGY_PART:
            return DataContainer._matsum
        elif name == ObjectConstantsForData.ENERGY_GLOBAL:
            return DataContainer._glstat
        elif name == ObjectConstantsForData.DISBOUT:
            return DataContainer._disbout
        elif name == ObjectConstantsForData.DISBOUT_PART:
            return DataContainer._part_disbout
        elif name == ObjectConstantsForData.AIRBAG:
            return DataContainer._abstat
        elif name == ObjectConstantsForData.AIRBAG_CPM:
            return DataContainer._abstat_cpm
        elif name == ObjectConstantsForData.SLEOUT:
            return DataContainer._sleout
        elif name == ObjectConstantsForData.RIGID_BODY:
            return DataContainer._rigid_body
        elif name == ObjectConstantsForData.BOUNDARY_CONDITION:
            return DataContainer._boundary_condition
        else:
            return None

    @staticmethod
    def init_all_data_sources(binout, logger, dynasaur_def, volume_path):
        """init all data sources

        Args:
          binout: 
          logger: 
          dynasaur_def: 
          volume_path: 

        Returns:

        """
        DataContainer._secforc = Secforc(binout, logger, dynasaur_def)
        DataContainer._elout = Elout(binout, logger, dynasaur_def)
        DataContainer._elout_object = EloutObject(binout, logger, dynasaur_def, volume_path=volume_path)
        DataContainer._nodout = Nodout(binout, logger, dynasaur_def)
        DataContainer._joint_forc = JointForc(binout, logger, dynasaur_def)
        DataContainer._joint_type = JointType(binout, logger, dynasaur_def)
        DataContainer._deforc = DEForc(binout, logger, dynasaur_def)
        DataContainer._disbout = Disbout(binout, logger, dynasaur_def)
        DataContainer._part_disbout = PartDisbout(binout, logger, dynasaur_def)
        DataContainer._rcforc = RCForc(binout, logger, dynasaur_def)
        DataContainer._sbtout = SBTout(binout, logger, dynasaur_def)
        DataContainer._slipringout = SlipringOut(binout, logger, dynasaur_def)
        DataContainer._matsum = Matsum(binout, logger, dynasaur_def)
        DataContainer._glstat = Glstat(binout, logger, dynasaur_def)
        DataContainer._abstat = Abstat(binout, logger, dynasaur_def)
        DataContainer._abstat_cpm = AbstatCPM(binout, logger, dynasaur_def)
        DataContainer._sleout = Sleout(binout, logger, dynasaur_def)
        DataContainer._rigid_body = RigidBody(binout, logger, dynasaur_def)
        DataContainer._boundary_condition = BoundaryCondition(binout, logger, dynasaur_def)




import time
import json

from lasso.dyna.Binout import Binout

from ..utils.constants import LOGConstants, DefinitionConstants, ObjectConstantsForData, JsonConstants, LoggerERROR, \
                              LoggerWARNING, PluginsParamDictDef, DataTypeMapping
from ..utils.constants import LoggerReadDynasaurDefinitionFile as LD
from ..utils.units import Units


class Helpers(object):
    """ """

    @staticmethod
    def create_default_object_definitions(data_source, code_type):
        """TODO
        documentation

        Args:
          data_source: param code_type:
          code_type: 

        Returns:

        """
        d = {"OBJECTS":[]}
        if code_type == "LS-DYNA":
            binout = Binout(data_source)
            for data_type in binout.read():
                if "ids" in binout.read(data_type):
                    legend_ids = [binout.read(data_type, "legend")[i:i + 80].strip(" ") for i in
                                  range(0, len(binout.read(data_type, "legend")), 80)]
                    if len(binout.read(data_type, "ids").shape) > 1:
                        ids = binout.read(data_type, "ids")[0]
                    else:
                        ids = binout.read(data_type, "ids")
                    for nr, id in enumerate(ids):
                        dynasaur_identifier = DataTypeMapping.binary_identifier_2_dynasaur_identifier(data_type)
                        if dynasaur_identifier is None:
                            print(data_type)
                            break
                        if len(legend_ids) == len(binout.read(data_type, "ids")):
                            d["OBJECTS"].append({"type": dynasaur_identifier, "name": legend_ids[nr], "id": [int(id)]})
                        else:
                            d["OBJECTS"].append({"type": dynasaur_identifier, "name": data_type + str(nr), "id": [int(id)]})

        return d


class DefintionObject(object):
    def __init__(self, oid):
        """Init function / constructor

        Args:
            oid: return:

        Returns:

        """
        self._oid = oid
        self._parts = {}
        self._elements = {}

    def set_parts(self, part):
        """

        Args:
          part: 

        Returns:

        """
        self._parts = part

    def add_part(self, pid):
        """

        Args:
          pid: 

        Returns:

        """
        self._parts[pid] = pid

    def add_element(self, eid):
        """

        Args:
          eid: 

        Returns:

        """
        self._elements[eid] = eid

    def get_parts(self):
        """ """
        return sorted(self._parts)

    def get_element(self):
        """ """
        return sorted(self._elements)


class DynasaurDefinitions:
    """ """
    # create the object container
    def __init__(self, logger):
        """
        constructor

        :param logger:
        :return:
        """
        self._title = ''
        self._objects = {}
        self._logger = logger
        self._info = []

        # set all class members
        self._dynasaur_data_type_container = {ObjectConstantsForData.__dict__[i]: {}
                                              for i in ObjectConstantsForData.__dict__ if not i.startswith("__")}

        self._criteria = {}
        self._data_vis = {}
        self._units = Units()

    def get_unit_dimension_from_unit(self, unit):
        """

        :param unit:
        :return:
        """
        ureg = self._units.ureg
        return ureg(unit)


    def get_param_dict_from_command(self, command):
        """

        Args:
          command: return dictionary on the command

        Returns:

        """
        if 'visualization' in command:
            d = command
            d[PluginsParamDictDef.DYNASAUR_JSON] = self.get_data_vis(command['visualization'])
            return d

        elif 'criteria' in command:
            d = command
            d[PluginsParamDictDef.DYNASAUR_JSON] = self.get_criteria(command['criteria'])
            return d
        else:
            self._logger.emit(LOGConstants.ERROR[0], "Command not in command list")
            return

    def get_units(self):
        """get units"""
        return self._units

    def get_criteria_ids(self):
        """ """
        return [key for key in self._criteria.keys()]

    def get_criteria(self, key):
        """get criteria

        Args:
          key: return criteria:

        Returns:

        """
        if key not in self._criteria.keys() or len(self._criteria) == 0:
            self._logger.emit(LOGConstants.ERROR[0], LoggerERROR.print_statements[6] + "\"" + key + "\"")
            return
        return self._criteria[key]

    def get_data_vis_calc_commands(self):
        """:return:"""

        ls = [key for key in self._data_vis.keys()]
        ret = []
        for key in ls:
            d = {"visualization": key}
            if "label" in self._data_vis[key]["x"]:
                x_label = self._data_vis[key]["x"]["label"]
                d["x_label"] = x_label
            if "label" in self._data_vis[key]["y"]:
                y_label = self._data_vis[key]["y"]["label"]
                d["y_label"] = y_label
            if "unit" in self._data_vis[key]["x"]:
                x_label = self._data_vis[key]["x"]["unit"]
                d["x_unit"] = x_label
            if "unit" in self._data_vis[key]["y"]:
                y_label = self._data_vis[key]["y"]["unit"]
                d["y_unit"] = y_label

            ret.append(d)
        return ret

    def get_criteria_calc_commands(self):
        """ """
        ls = [key for key in self._criteria.keys()]
        ret = []
        for nr, key in enumerate(ls):
            ret.append({"criteria": key})

        return ret

    def get_data_vis(self, key):
        """get code

        Args:
          key: 

        Returns:

        """
        if key not in self._data_vis.keys() or len(self._data_vis) == 0:
            self._logger.emit(LOGConstants.ERROR[0], LoggerERROR.print_statements[7] + "\"" + key + "\"")
        return self._data_vis[key]

    def get_object_ids_from_object_name(self, object_name, data_container_name):
        """get ids from given name

        Args:
          object_name: param data_container:
          data_container_name:

        Returns:

        """

        dynasaur_data_types = DataTypeMapping.binary_identifier_2_dynasaur_identifier(data_container_name)
        ret = []
        for dynasaur_data_type in dynasaur_data_types:
            d = self._dynasaur_data_type_container[dynasaur_data_type]
            if object_name not in d.keys() or len(d) == 0:
                continue
            ret = d[object_name]

        if len(ret) == 0:
            self._logger.emit(LOGConstants.ERROR[0], "\"" + object_name + "\" is not defined for " + ", ".join(dynasaur_data_types))

        return ret

    def get_defined_objects_containing_parts(self, part_ids):
        """get defined object containing parts

        Args:
          part_ids: return list_defined_objects:

        Returns:

        """
        list_defined_objects = []
        for tmp_object_name in sorted(self._objects):
            tmp_part_ids = list(self._objects[tmp_object_name].get_parts())
            intersection = [val for val in tmp_part_ids if val in part_ids]
            if len(intersection) != 0:
                list_defined_objects.append(tmp_object_name)

        if len(list_defined_objects) == 0:
            list_defined_objects.append("")

        return list_defined_objects

    def get_defined_objects_containing_all_parts(self, part_ids):
        """get defined object

        Args:
          part_ids: return list_defined_objects:

        Returns:

        """
        list_defined_objects = []
        for tmp_object_name in sorted(self._objects):
            tmp_part_ids = list(self._objects[tmp_object_name].get_parts())
            intersection = [val for val in tmp_part_ids if val in part_ids]
            if len(tmp_part_ids) == len(intersection):
                list_defined_objects.append(tmp_object_name)

        if len(list_defined_objects) == 0:
            list_defined_objects.append("")

        return list_defined_objects

    def define_dynasaur_everything(self, part_ids):
        """define dynasaur everything

        Args:
          part_ids: list of part_ids

        Returns:

        """

        # create an Everything object
        name = "Dynasaur Everything"
        self._objects[name] = DefintionObject(name)

        # only store ids not the entire dict would cause side effects
        self._objects[name].set_parts(part_ids)

    def get_all_data_types_from_json(self, d, ls):
        """get all data types

        Args:
          d: param ls:
        :return d[TYPE]:
          ls: 

        Returns:

        """

        for key in d.keys():
            if key is None:
                continue
            if isinstance(d[key], dict):
                data_type = self.get_all_data_types_from_json(d[key], ls)
                if data_type is not None and data_type not in ls:
                    if isinstance(data_type, list):
                        for type_ in data_type:
                            ls.append(type_)
                    else:
                        ls.append(data_type)
            elif JsonConstants.TYPE in d.keys():
                return d[JsonConstants.TYPE]

    def get_required_datatypes(self, plugin_name):
        """get required datatype

        Args:
          plugin_name: return ls:

        Returns:

        """
        ls = []
        d = {}
        if DefinitionConstants.CRITERIA == plugin_name:
            d = self._criteria
        elif DefinitionConstants.DATA_VIS == plugin_name:
            d = self._data_vis

        self.get_all_data_types_from_json(d, ls)

        return ls

    def get_parts_by_object_containing_part_ids(self, tmp_object, part_ids):
        """get parts by object containing part ids

        Args:
          tmp_object: param part_ids:
        :return parts by object:
          part_ids: 

        Returns:

        """
        return [val for val in part_ids if val in self._objects[tmp_object].get_parts()]

    def get_parts_of_defined_object(self, tmp_object):
        """

        Args:
          tmp_object: type  str:

        Returns:
          array:: the parts as defined in ids.def:

        """
        return self._objects[tmp_object].get_parts()

    def get_info(self):
        """get info"""
        return self._info

    def _set_contact(self, json_object):
        """set contact object type

        Args:
          json_object: return:

        Returns:

        """
        name = json_object[JsonConstants.NAME]

        if JsonConstants.ID in json_object:
            self._dynasaur_data_type_container["CONTACT"][name] = json_object[JsonConstants.ID]

        if JsonConstants.ID_RANGE in json_object:
            self._dynasaur_data_type_container["CONTACT"][name] = [str(i) + json_object[JsonConstants.ID_RANGE][0][-1]
                                   for i in range(int(json_object[JsonConstants.ID_RANGE][0][:-1]),
                                                  int(json_object[JsonConstants.ID_RANGE][-1][:-1]) + 1)]

    def _set_datacontainer_defintion(self, json_object, def_container):
        """

        Args:
          json_object: param def_container:
          def_container: 

        Returns:

        """
        name = json_object[JsonConstants.NAME]

        if JsonConstants.ID in json_object:
            def_container[name] = json_object[JsonConstants.ID]

        if JsonConstants.ID_RANGE in json_object:
            def_container[name] = [i for i in range(json_object[JsonConstants.ID_RANGE][0],
                                                    json_object[JsonConstants.ID_RANGE][-1] + 1)]

        if JsonConstants.PART_ID in json_object:
            def_container[name] = json_object[JsonConstants.PART_ID]

    def is_object_defined(self, tmp_object):
        """
        checks if the tmp_object is defined in the objects.def file

        Args:
          tmp_object: ID string as defined in the calc_proc.def file
          (i.e. object_data": {
                            "type" : "OBJECT",
                            "ID" : "Ribs",
                            "strain_stress": "Strain"})

        Returns:
            True or False
        """

        return tmp_object in self._objects

    def __set_object(self, json_object):
        """
        set object

        :param json_object:
        :return:
        """
        name = json_object[JsonConstants.NAME]
        self._objects[name] = DefintionObject(name)
        if JsonConstants.ID in json_object:
            for part in json_object[JsonConstants.ID]:
                self._objects[name].add_part(part)

        if JsonConstants.ID_RANGE in json_object:
            for i in range(json_object[JsonConstants.ID_RANGE][0], json_object[JsonConstants.ID_RANGE][-1]):
                self._objects[name].add_part(i)

        # TODO: Check if we need elem?
        elif "elem" in json_object:
            for elem in json_object["elem"]:
                self._objects[name].add_element(elem)

        elif "elem_range" in json_object:
            for i in range(json_object["elem_range"][0], json_object["elem_range"][1]):
                self._objects[name].add_element(i)

    def _offset_ids(self, offset):
        """update id on offset

        Args:
          json_objects:

        Returns:

        """
        for object_name in offset:
            for key_ in self._dynasaur_data_type_container[object_name]:
                for index, value_ in enumerate(self._dynasaur_data_type_container[object_name][key_]):
                    if object_name == ObjectConstantsForData.CONTACT:
                        offset_value = int(value_[:-1]) + offset[object_name]
                        self._dynasaur_data_type_container[object_name][key_][index] = str(offset_value) + value_[-1]
                    else:
                        assert isinstance(value_, int) and isinstance(offset[object_name], int)
                        self._dynasaur_data_type_container[object_name][key_][index] = value_ + offset[object_name]

    def _parse_objects(self, json_objects):
        """parse json objects and call setter

        Args:
          json_object: return:
          json_objects: 

        Returns:

        """
        for json_object in json_objects[DefinitionConstants.OBJECTS]:

            if json_object[JsonConstants.TYPE] not in self._dynasaur_data_type_container.keys():
                self._logger.emit(LOGConstants.WARNING[0],
                                  json_object[JsonConstants.TYPE] + LoggerWARNING.print_statements[1] +
                                  str(", ".join(list(self._dynasaur_data_type_container.keys()))))
                continue

            if json_object[JsonConstants.TYPE] == ObjectConstantsForData.ELEMENTOBJECT:
                self.__set_object(json_object)
            elif json_object[JsonConstants.TYPE] == ObjectConstantsForData.CONTACT:
                self._set_contact(json_object)
            else:
                data_type_container = self._dynasaur_data_type_container[json_object[JsonConstants.TYPE]]
                self._set_datacontainer_defintion(json_object, data_type_container)

        if DefinitionConstants.OFFSET in json_objects:
            self._offset_ids(json_objects[DefinitionConstants.OFFSET])

    def _set_criteria(self, json_object):
        """set criteria object

        Args:
          json_object: return:

        Returns:

        """
        for criteria in json_object[DefinitionConstants.CRITERIA]:
            body_part = criteria[JsonConstants.PART_OF]
            self._criteria[body_part + "_" + criteria[JsonConstants.NAME]] = criteria

    def _set_diagram_visualisation(self, json_object):
        """set data visualisation object

        Args:
          json_object: return:

        Returns:

        """
        for definition in json_object[DefinitionConstants.DATA_VIS]:
            body_part = definition[JsonConstants.PART_OF]
            if not body_part + "_" + definition[JsonConstants.NAME] in self._data_vis:
                self._data_vis[body_part + "_" + definition[JsonConstants.NAME]] = definition
            else:
                self._logger.emit(LOGConstants.WARNING[0], "Double declaration of " + body_part + "_" + definition[JsonConstants.NAME])

    def _set_info(self):
        """set info in data visualisation"""
        info = []
        for key in self._data_vis.keys():
            if JsonConstants.INFO in self._data_vis[key].keys():
                if len(self._data_vis[key][JsonConstants.INFO].split(":")) == 3:
                    info.append(self._data_vis[key][JsonConstants.INFO])
                elif len(self._data_vis[key][JsonConstants.INFO].split(":")) == 2:
                    self._data_vis[key][JsonConstants.INFO].append(":")
                    info.append(self._data_vis[key][JsonConstants.INFO])
                elif len(self._data_vis[key][JsonConstants.INFO].split(":")) > 3:
                    self._data_vis[key][JsonConstants.INFO] = "::"
                    info.append(self._data_vis[key][JsonConstants.INFO])
                else:
                    self._data_vis[key][JsonConstants.INFO].append(":")
                    info.append(self._data_vis[key][JsonConstants.INFO])
            else:
                info.append(None)
        self._info = info

    def read_def(self, fn):
        """read definition file(s)

        Args:
          fn: return:

        Returns:

        """
        if fn is None:
            self._logger.emit(LOGConstants.ERROR[0], "No definition file")
            return

        if isinstance(fn, str):
            fn = [fn]

        if isinstance(fn, list):

            if len(fn) < 1:
                self._logger.emit(LOGConstants.ERROR[0], "No definition file(s) in list")
                return
            is_entity_set = {i: False for i in DefinitionConstants.__dict__ if i[:1] != "_"}

            for single_def_file in fn:
                self._logger.emit(LOGConstants.READ_DYNASAUR_DEF[0], LD.READ % single_def_file)
                try:
                    with open(single_def_file) as fd:
                        try:
                            json_objects = json.load(fd)
                            for j_object in json_objects:
                                if DefinitionConstants.TITLE in j_object:
                                    if not is_entity_set[DefinitionConstants.TITLE]:
                                        self._title = j_object[DefinitionConstants.TITLE]
                                        is_entity_set[DefinitionConstants.TITLE] = True
                                    else:
                                        self._logger.emit(LOGConstants.ERROR[0], LoggerERROR.print_statements[8] + DefinitionConstants.TITLE)
                                        return

                                elif DefinitionConstants.UNIT in j_object:
                                    if not is_entity_set[DefinitionConstants.UNIT]:
                                        self._units = Units(**j_object[DefinitionConstants.UNIT])
                                        is_entity_set[DefinitionConstants.UNIT] = True
                                    else:
                                        self._logger.emit(LOGConstants.ERROR[0], LoggerERROR.print_statements[8] + DefinitionConstants.UNIT)
                                        return

                                elif DefinitionConstants.CRITERIA in j_object:
                                    self._set_criteria(j_object)

                                elif DefinitionConstants.DATA_VIS in j_object:
                                    self._set_diagram_visualisation(j_object)
                                    self._set_info()

                                elif DefinitionConstants.OBJECTS in j_object:
                                    self._parse_objects(json_objects)

                                elif DefinitionConstants.OFFSET in j_object:
                                    pass

                                else:
                                    self._logger.emit(LOGConstants.ERROR[0],
                                                      "json object not understood:" + str(j_object))
                                    return

                        except ValueError as e:
                            self._logger.emit(LOGConstants.ERROR[0], LoggerERROR.print_statements[4] % e)
                            self._logger.emit(LOGConstants.ERROR[0], "Invalid JSON file")
                            return
                except FileNotFoundError as e:
                    self._logger.emit(LOGConstants.ERROR[0],  e.strerror + " " + e.filename)
                    return

        else:
            self._logger.emit(LOGConstants.ERROR[0], "No definition file")
            return

        self._logger.emit(LOGConstants.READ_DYNASAUR_DEF[0], LD.DONE)

